﻿using System;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Linq;

namespace ElmarBoye.Samples
{
    /// <summary>
    /// Speichert die Einstellungen für ein DataGridView
    /// </summary>
    /// <remarks>
    /// Konzeptionell auf weitere Steuerelemente anwendbar.
    /// </remarks>
    public class GridEinstellungen : IDisposable
    {
        private GridEinstellungenAdapter _adapter;

        /// <summary>
        /// Erstellt einen Adapter mit der angegeben Verbindungszeichenfolge.
        /// </summary>
        /// <param name="connectionString"></param>
        public GridEinstellungen(string connectionString)
        {
            this._adapter = new GridEinstellungenAdapter(connectionString);
        }

        /// <summary>
        /// Freigabe des Adapters.
        /// </summary>
        public void Dispose()
        {
            if (this._adapter != null)
            {
                this._adapter.Dispose();
                this._adapter = null;
            }
        }

        /// <summary>
        /// Speichert das Layout für ein DataGridView unter ein Benutzerkennung
        /// </summary>
        public void SaveLayout(int personId, DataGridView gridView)
        {
            // Formularname als Pfad über FormularName.Container
            string formularName = GetFormularName(gridView);
            var layout = GridLayoutToXml(gridView).ToString();

            // Abrufen des Layouts für den Benutzer (wenn bereits vorhanden)
            var table = this._adapter.GetData(personId, GetFormularName(gridView));

            // Wenn neu, Anlage der Einstellungen sonst abrufen
            if (table.Rows.Count == 0)
                table.Rows.Add(personId, formularName, layout);
            else
                table.Rows[0]["Einstellungen"] = layout;

            // Speichern
            this._adapter.Update(table);
        }

        /// <summary>
        /// Lädt ein zuvor´gespeichertes DataGridView-Layout
        /// </summary>
        public void LoadLayout(int personId, DataGridView gridView)
        {
            // TODO: Standard-Layout für alle Benutzer (z. B. PersonID = 0)

            // Formularname als Pfad über FormularName.Container
            string formularName = GetFormularName(gridView);

            // Liefert nur eine Zeile
            var table = this._adapter.GetData(personId, GetFormularName(gridView));
            // Daten vorhanden?
            if (table.Rows.Count == 1)
            {
                // Konvertieren in XDocument
                var layout = XDocument.Parse((string)table.Rows[0]["Einstellungen"]);

                GridLayoutFromXml(gridView, layout);
            }
        }

        /// <summary>
        /// Erstellen Eigenschaften der DataGridView.Column Auflistung als Xml
        /// </summary>
        public static XDocument GridLayoutToXml(DataGridView gridView)
        {
            // Abfragen aller Column-Auflistung und Ausgabe ausgewählter Eigenschaften
            // TODO: Erweitern um abgeleitete DataGridViewColumn sofern erforderlich
            var columnQuery =
                from column in gridView.Columns.Cast<DataGridViewColumn>()
                select new XElement("Column",
                    new XAttribute("ColumnName", column.Name),
                    new XAttribute("DisplayIndex", column.DisplayIndex),
                    new XAttribute("Visible", column.Visible),
                    new XAttribute("Width", column.Width));

            // Erstellen Element für das DataGridView
            var document = new XDocument(
                new XElement("Einstellungen",
                    new XElement("Control",
                        new XAttribute("ControlName", gridView.Name),
                        new XElement("Columns", columnQuery.ToArray())
                     )
                ));
            return document;
        }

        /// <summary>
        /// Wendet die Xml-Eigenschaften auf die DataGridView.Column Auflistung an.
        /// </summary>
        public static void GridLayoutFromXml(DataGridView gridView, XDocument layout)
        {
            gridView.SuspendLayout();

            // XML Query => http://msdn.microsoft.com/de-de/library/bb943906.aspx
            var columns =
                from c in layout.Root.Element("Control").Element("Columns").Elements("Column")
                orderby (string)c.Attribute("DisplayIndex")
                select new
                {
                    ColumnName = (string)c.Attribute("ColumnName"),
                    DisplayIndex = (int)c.Attribute("DisplayIndex"),
                    Visible = (bool)c.Attribute("Visible"),
                    Width = (int)c.Attribute("Width")
                };

            // Übertragen der Eigenschaften.
            foreach (var column in columns)
            {
                // Noch vorhanden?
                if (gridView.Columns.Contains(column.ColumnName))
                {
                    var gridColumn = gridView.Columns[column.ColumnName];
                    gridColumn.DisplayIndex = column.DisplayIndex;
                    gridColumn.Visible = column.Visible;
                    gridColumn.Width = column.Width;
                }
            }

            gridView.ResumeLayout();
        }

        /// <summary>
        /// Ermittelt den Formularnnamen inkl. der untergeordneten Container
        /// um eine eindeutige Bezeichnung zu erhalten.
        /// </summary>
        public static string GetFormularName(Control control)
        {
            var builder = new StringBuilder();
            var parent = control.Parent;
            while (parent != null)
            {
                if (builder.Length != 0)
                    builder.Insert(0, '.');
                builder.Insert(0, parent.Name);

                parent = parent.Parent;
            }
            return builder.ToString();
        }
    }
}
