﻿using System;
using System.Data;
using System.Data.SqlClient;

namespace ElmarBoye.Samples
{
    /// <summary>
    /// DataApdater für <see cref="GridEinstellungen"/>
    /// </summary>
    /// <remarks>
    /// Hier reduziert auf SqlDataAdapter, kann auch via TableAdapter typisiert geschehen.
    /// Erstellt hier die Tabelle beim Aufruf sofern erforderlich
    /// </remarks>
    public class GridEinstellungenAdapter : IDisposable
    {
        private const string TableName = "GridEinstellungen";
        private SqlDataAdapter _adapter;

        public GridEinstellungenAdapter(string connectionString)
        {
            // Für Beispiel erforderlich - (hier inline, kann später entfallen)
            CreateTable(connectionString);

            // Erstellen des Adapters
            this._adapter = CreateAdapter(connectionString);
        }

        public void Dispose()
        {
            // Verkürzt, siehe http://msdn.microsoft.com/de-de/library/b1yfkh5e.aspx
            if (this._adapter != null)
            {
                this._adapter.Dispose();
                this._adapter = null;
            }
        }

        /// <summary>Abrufen einer DataTable für die Person und das Formular</summary>
        public DataTable GetData(int personId, string formularName)
        {
            var table = new DataTable(TableName);
            _adapter.SelectCommand.Parameters["@PersonId"].Value = personId;
            _adapter.SelectCommand.Parameters["@FormularName"].Value = formularName;
            try
            {
                this._adapter.Fill(table);
                return table;
            }
            catch (SqlException ex)
            {
                throw new DataException("Abruf fehlgeschlagen", ex);
            }
        }

        /// <summary>Aktualisieren der Einstellungen.</summary>
        public int Update(DataTable table)
        {
            try
            {
                return this._adapter.Update(table);
            }
            catch (SqlException ex)
            {
                throw new DataException("Abruf fehlgeschlagen", ex);
            }
        }


        /// <summary>Erstellt einen SqlDataAdapter für die Tabelle GridEinstellungen.</summary>
        private static SqlDataAdapter CreateAdapter(string connectionString)
        {
            var adapter = new SqlDataAdapter(
                "SELECT PersonId, FormularName, Einstellungen\r\n"
                + "FROM dbo.GridEinstellungen\r\n"
                + "WHERE PersonId=@PersonId AND FormularName=@FormularName;",
                connectionString);
            adapter.SelectCommand.Parameters.Add("@PersonID", SqlDbType.Int, 4, "PersonId");
            adapter.SelectCommand.Parameters.Add("@FormularName", SqlDbType.NVarChar, 128, "FormularName");

            // InsertCommand
            adapter.InsertCommand = new SqlCommand(
                "INSERT INTO dbo.GridEinstellungen (PersonID, FormularName, Einstellungen)\r\n"
                + "VALUES(@PersonID, @FormularName, @Einstellungen);",
                adapter.SelectCommand.Connection);
            adapter.InsertCommand.Parameters.Add("@PersonID", SqlDbType.Int, 4, "PersonId");
            adapter.InsertCommand.Parameters.Add("@FormularName", SqlDbType.NVarChar, 128, "FormularName");
            adapter.InsertCommand.Parameters.Add("@Einstellungen", SqlDbType.NVarChar, -1, "Einstellungen");

            // UpdateCommand
            adapter.UpdateCommand = new SqlCommand(
                "UPDATE dbo.GridEinstellungen SET\r\n"
                + "PersonID=@PersonId, FormularName=@FormularName, Einstellungen=@Einstellungen\r\n"
                + "WHERE PersonId=@PersonId_Original AND FormularName=@FormularName_Original;",
                adapter.SelectCommand.Connection);
            adapter.UpdateCommand.Parameters.Add("@PersonID", SqlDbType.Int, 4, "PersonId");
            adapter.UpdateCommand.Parameters.Add("@FormularName", SqlDbType.NVarChar, 128, "FormularName");
            adapter.UpdateCommand.Parameters.Add("@Einstellungen", SqlDbType.NVarChar, -1, "Einstellungen");

            adapter.UpdateCommand.Parameters.Add("@PersonID_Original", SqlDbType.Int, 4, "PersonId")
                .SourceVersion = DataRowVersion.Original;
            adapter.UpdateCommand.Parameters.Add("@FormularName_Original", SqlDbType.NVarChar, 128, "FormularName")
                .SourceVersion = DataRowVersion.Original;

            //adapter.ReturnProviderSpecificTypes = true;
            adapter.MissingMappingAction = MissingMappingAction.Passthrough;
            adapter.MissingSchemaAction = MissingSchemaAction.AddWithKey;
            return adapter;
        }

        /// <summary>Erstellen der Beispieltabelle (nur für Beispiel erforderlich).</summary>
        public static void CreateTable(string connectionString)
        {
            using (var connection = new SqlConnection(connectionString))
            {
                connection.Open();

                var createTableCommand = new SqlCommand(
                "IF OBJECT_ID(N'dbo.GridEinstellungen', 'U') IS NULL\r\n"
                 + "CREATE TABLE dbo.GridEinstellungen (\r\n"
                 + "PersonId int NOT NULL,\r\n"
                 + "FormularName nvarchar(128) NOT NULL,\r\n"
                    // Hier NVARCHAR(MAX) alternativ XML (oder NTEXT für SQL 2000)
                 + "Einstellungen nvarchar(MAX) NOT NULL,\r\n"
                 + "CONSTRAINT PK_GridSettings PRIMARY KEY (PersonId, FormularName));",
                connection);

                createTableCommand.ExecuteNonQuery();
            }
        }
    }
}
